@@ -1,4 +1,4 @@
-# Copyright (c) 2014 Marty Sullivan
+# Copyright (c) 2015 Marty Sullivan
 # 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,8 +23,6 @@
 	NDFD Forecast Retrieval Routines
 
 	Author: 	Marty J. Sullivan
-	Revision: 	0.1
-	Date:		December 3, 2014
 	Purpose:	Routines that will cache NDFD forecast variables locally
 			to allow for easy and fast forecast analysis by lat/lon
 
@@ -36,19 +34,19 @@
 #         #
 ###########
 
-from bitstring import BitArray
 from datetime import datetime, timedelta
+from getpass import getuser
 from math import isnan, sqrt
 from ncepgrib2 import Grib2Decode as ncepgrib
 from ndfd_defs import ndfdDefs
 from numpy.ma.core import MaskedConstant as NAN
 from os import makedirs, path
 from pyproj import Geod, Proj
+from shutil import rmtree
 from sys import stderr
 from tempfile import gettempdir
 from urllib import urlretrieve
 import json
-
 import pygrib
 
 #############
@@ -60,14 +58,14 @@ import pygrib
 DEFS = ndfdDefs()
 G = Geod(ellps='clrk66')
 
-CACHE_SERVER_BUFFER_MIN = 10
+CACHE_SERVER_BUFFER_MIN = 20
 
 NDFD_LOCAL_SERVER = None
-NDFD_REMOTE_SERVER = 'http://weather.noaa.gov/pub/SL.us008001/ST.opnl/DF.gr2/'
+NDFD_REMOTE_SERVER = 'http://tgftp.nws.noaa.gov/SL.us008001/ST.opnl/DF.gr2/'
 NDFD_DIR = 'DC.ndfd' + path.sep + 'AR.{0}' + path.sep + 'VP.{1}' + path.sep
 NDFD_STATIC = 'static' + path.sep + 'DC.ndfd' + path.sep + 'AR.{0}' + path.sep
 NDFD_VAR = 'ds.{0}.bin'
-NDFD_TMP = gettempdir() + path.sep
+NDFD_TMP = gettempdir() + path.sep + str(getuser()) + '_pyndfd' + path.sep
 
 ########################
 #                      #
@@ -99,7 +97,7 @@ def stdDev(vals):
     mean = sum(vals) / len(vals)
     squared = []
     for val in vals:
-        squared.append(pow(val - mean, 2))
+        squared.append(pow(val - mean, 2) * 1.0)
     variance = sum(squared) / len(squared)
     return sqrt(variance)
 
@@ -147,6 +145,8 @@ def getVariable(var, area):
     gribs = []
     dirTime = NDFD_TMP + getLatestForecastTime().strftime('%Y-%m-%d-%H') + path.sep
     if not path.isdir(dirTime):
+        try: rmtree(NDFD_TMP)
+        except: pass
         makedirs(dirTime)
     if area in DEFS['vars']:
         for vp in DEFS['vars'][area]:
@@ -415,6 +415,7 @@ def getForecastAnalysis(var, lat, lon, n=0, timeStep=1, elev=False, minTime=None
                 forecast['mean'] = sum(vals) / len(vals)
                 forecast['median'] = median(vals)
                 forecast['stdDev'] = stdDev(vals)        
+                forecast['sum'] = sum(vals)
 
             if elev:
                 elevation = { }
@@ -434,11 +435,20 @@ def getForecastAnalysis(var, lat, lon, n=0, timeStep=1, elev=False, minTime=None
             analysis['forecasts'][t] = forecast
         grbs.close()
 
-    analysis['min'] = min(allVals)
-    analysis['max'] = max(allVals)
-    analysis['mean'] = sum(allVals) / len(allVals)
-    analysis['median'] = median(allVals)
-    analysis['stdDev'] = stdDev(allVals)
+    analysis['min'] = float('nan')
+    analysis['max'] = float('nan')
+    analysis['mean'] = float('nan')
+    analysis['median'] = float('nan')
+    analysis['stdDev'] = float('nan')
+    analysis['sum'] = float('nan')
+
+    if len(allVals) > 0:
+        analysis['min'] = min(allVals)
+        analysis['max'] = max(allVals)
+        analysis['mean'] = sum(allVals) / len(allVals)
+        analysis['median'] = median(allVals)
+        analysis['stdDev'] = stdDev(allVals)
+        analysis['sum'] = sum(allVals)
 
     return analysis
 
@@ -448,27 +458,25 @@ def getForecastAnalysis(var, lat, lon, n=0, timeStep=1, elev=False, minTime=None
   Purpose:	To unpack the packed binary string in the local use section of NDFD gribs
   Params:
 	raw:	The raw byte string containing the packed data
-  Notes:
-	- This function is pretty slow and could be optimized with a C routine.
-
 '''
+
 def unpackString(raw):
-    msg = ''
+    num_bytes, remainder = divmod(len(raw) * 8 - 1, 7)
 
-    bits = BitArray(bytes=raw)
-    mask = BitArray('0b01111111')
+    i = int(raw.encode('hex'), 16)
+    if remainder:
+        i >>= remainder
 
-    i = 0
-    while 1:
-        try:
-            iByte = (bits[i:i + 8] & mask).int
-            if iByte == 0:
-                msg += '\n'
-            elif iByte >= 32 and iByte <= 126:
-                msg += chr(iByte)
-            i += 7
-        except:
-            break
+    msg = []
+    for _ in range(num_bytes):
+        byte = i & 127
+        if not byte:
+            msg.append(ord("\n"))
+        elif 32 <= byte <= 126:
+            msg.append(byte)
+        i >>= 7
+    msg.reverse()
+    msg = b"".join(chr(c) for c in msg)
 
     codes = []
     for line in msg.splitlines():
@@ -532,16 +540,16 @@ def parseWeatherString(wxString):
             ws += 'likely '
 
         for attribute in attributes:
-            if len(attribute) == 0 or '<None>' in attribute:
+            if len(attribute) == 0 or '<None>' in attribute or 'Mention' in attribute:
                 pass
             elif attribute == 'Primary':
                 prepend = True
             elif attribute == 'OR':
                 OR = True
             elif attribute in DEFS['wx']['hazards']:
-                ws += 'with ' + DEFS['wx']['hazards'][attribute]
+                ws += 'with ' + DEFS['wx']['hazards'][attribute] + ' '
             elif attribute in DEFS['wx']['attributes']:
-                ws += DEFS['wx']['attributes'][attribute]
+                ws += DEFS['wx']['attributes'][attribute] + ' '
             else:
                 stderr.write('WARNING: Unknown attribute code: ' + attribute + '\n'); stderr.flush()
 
